import 'dart:ffi';
import 'dart:typed_data';
import 'package:ffi/ffi.dart';

class Int32Array extends Struct {
  external Pointer<Int32> list;

  @Int64()
  external int size;
}

class Int64Array extends Struct {
  external Pointer<Int64> list;

  @Int64()
  external int size;
}

class DString extends Struct {
  external Pointer<Uint8> content;

  @Int64()
  external int size;

  factory DString.allocate(Pointer<Uint8> content, int size) =>
      malloc.allocate<DString>(1).ref
        ..content = content
        ..size;
}

// bytes数组类型，包括Protobuf类型
class DStringList extends Struct {
  external Pointer<Pointer<DString>> list;

  @Int64()
  external int size;
}

// 字符串数组类型
class StringList extends Struct {
  external Pointer<Pointer<Utf8>> list;

  @Int64()
  external int size;

  Pointer<StringList> fromList(List<String> arr) {
    final Pointer<Pointer<Utf8>> list = malloc.allocate(arr.length);

    for (var i = 0; i < arr.length; i++) {
      list[i] = arr.elementAt(i).toNativeUtf8();
    }

    final pStrList = malloc<StringList>();

    pStrList.ref.list = list;
    pStrList.ref.size = arr.length;

    return pStrList;
  }
}

class PrimitiveWrapper {
  var value;
  PrimitiveWrapper(this.value);
}

// String to Uint8List
Uint8List convertStringToUint8List(String str) {
  final List<int> codeUnits = str.codeUnits;
  final Uint8List unit8List = Uint8List.fromList(codeUnits);

  return unit8List;
}

String convertListIntToString(List<int> list) {
  final Uint8List uint8list = Uint8List.fromList(list);
  return String.fromCharCodes(uint8list);
}
// Uint8List to String
String convertUint8ListToString(Uint8List uint8list) {
  return String.fromCharCodes(uint8list);
}
// Uint8List to Pointer<Uint8>
Pointer<Uint8> createUint8PointerFromUint8List(Uint8List data) {
  final Pointer<Uint8> nativeUint8Buffer =
  malloc.allocate<Uint8>(data.length + 1);
  for (int i = 0; i < data.length; ++i) {
    nativeUint8Buffer.elementAt(i).value = data.elementAt(i);
  }
  return nativeUint8Buffer;
}

Uint8List createUint8ListFromPointer(Pointer<Uint8> data, int len) {
  Uint8List list = Uint8List(len);
  final resByteData = list.buffer.asByteData();
  for (int i = 0; i < list.length; ++i) {
    resByteData.setUint8(i, data.elementAt(i).value);
  }
  return list;
}
// protobuf object to Pointer<Uint8>
Pointer<Uint8> createPbDStringPointer(dynamic data) {
  Uint8List uint8list = data.writeToBuffer();
  Pointer<Uint8> uint8Pointer = createUint8PointerFromUint8List(uint8list);
  return uint8Pointer;
}